//
//  AFURLRequest.m
//  AFURLRequester
//
//  Created by Arkady on 27.03.11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "AFURLRequest.h"
#import "NSString+AFURLRequest.h"

NSString *const AFURLRequestErrorDomain = @"com.fraerman.afurlrequest";

@interface AFURLRequest ()
{
    float _totalDataLength;
    float _loadedDataLength;
    AFURLRequestCompletionBlock _completeBlock;
}

@property (nonatomic, strong) NSMutableData *activeDownload;
@property (nonatomic, strong) NSURLConnection *apiConnection;

@end



@implementation AFURLRequest

#pragma mark - Custom getters/setters

- (NSMutableURLRequest *)request
{
    if (_request == nil)
        _request = [self defaultRequest];
    return _request;
}



#pragma mark - Initializatoin

- (id)init
{
    self = [super init];
    if (self) {
        [self setup];
    }
    return self;
}



#pragma mark - Buisness

- (void)setup
{
    // defaults
    self.urlString = @"";
    self.GETParameters = nil;
    self.POSTParameters = nil;
    _totalDataLength = 0.0f;
    _loadedDataLength = 0.0f;
}



- (void)cancel
{
    if (self.apiConnection == nil)
		return;
    [self.apiConnection cancel];
    self.apiConnection = nil;
    self.activeDownload = nil;
	[[AFActivityIndicatorManager sharedActivityIndicatorManager] decreaseActivityCounter];
    _completeBlock(nil, nil);
}



- (NSMutableURLRequest *)defaultRequest
{
    NSMutableString *fullUrl = [NSMutableString stringWithString:self.urlString];
    
    // add get vars
    if (self.GETParameters != nil)
        [fullUrl appendFormat:@"?%@", [self serializeParameters:self.GETParameters]];
    
    NSURL *requestURL = [NSURL URLWithString:fullUrl];
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:requestURL];
    
    // add POST vars
    if (self.POSTParameters != nil) {
        NSString *postString = [self serializeParameters:self.POSTParameters];
        [request setHTTPMethod:@"POST"];
        [request setValue:@"application/x-www-form-urlencoded" forHTTPHeaderField:@"Content-Type"];
        [request setHTTPBody:[postString dataUsingEncoding:NSUTF8StringEncoding allowLossyConversion:YES]];
        [request setValue:[NSString stringWithFormat:@"%d", [postString length]] forHTTPHeaderField:@"Content-Length"];
    }
    
    NSLog(@"AFRURLReqest. Request url: %@", fullUrl);
    return request;
}



- (void)requestWithComplete:(AFURLRequestCompletionBlock)complete
{
    _completeBlock = complete;
    
    // Check network status
    if ([AFNetworkChecker getNetworkStatus] == AFNetworkNotReachable) {
        NSString *errorDescription = [[NSBundle mainBundle] localizedStringForKey:@"Network connection is not reachable"
                                                                            value:@"Network connection is not reachable"
                                                                            table:@"error"];
        _completeBlock(nil, [NSError errorWithDomain:AFURLRequestErrorDomain
                                                code:1
                                            userInfo:@{NSLocalizedDescriptionKey: errorDescription}]);
        return;
    }
    
    if (self.apiConnection)
        [self.apiConnection cancel];
    [[AFActivityIndicatorManager sharedActivityIndicatorManager] increaseActivityCounter];
    self.activeDownload = [NSMutableData data];
    NSURLConnection *conn = [[NSURLConnection alloc] initWithRequest:self.request delegate:self];
    self.apiConnection = conn;
}



- (void)uploadData:(NSData *)data complete:(AFURLRequestCompletionBlock)complete
{
    _completeBlock = complete;
    
    // Check network status
    if ([AFNetworkChecker getNetworkStatus] == AFNetworkNotReachable) {
        NSString *errorDescription = [[NSBundle mainBundle] localizedStringForKey:@"Network connection is not reachable"
                                                                            value:@"Network connection is not reachable"
                                                                            table:@"error"];
        _completeBlock(nil, [NSError errorWithDomain:AFURLRequestErrorDomain
                                                code:1
                                            userInfo:@{NSLocalizedDescriptionKey: errorDescription}]);
        return;
    }
    
    if (self.apiConnection)
        [self.apiConnection cancel];
    
    NSMutableURLRequest *request = [[NSMutableURLRequest alloc] init];
    [request setURL:[NSURL URLWithString:self.urlString]];
    [request setHTTPMethod:@"POST"];
    
    NSString *boundary = @"---------------------------14737809831466499882746641449";
    NSString *contentType = [NSString stringWithFormat:@"multipart/form-data; boundary=%@", boundary];
    [request addValue:contentType forHTTPHeaderField: @"Content-Type"];
    
    NSMutableData *body = [NSMutableData data];
    [body appendData:[[NSString stringWithFormat:@"\r\n--%@\r\n",boundary] dataUsingEncoding:NSUTF8StringEncoding]];
    NSString *disposition = @"Content-Disposition: form-data; name=\"photo\"; filename=\"photo.jpg\"\r\n";
    [body appendData:[disposition dataUsingEncoding:NSUTF8StringEncoding]];
    NSString *type = @"Content-Type: application/octet-stream\r\n\r\n";
    [body appendData:[type dataUsingEncoding:NSUTF8StringEncoding]];
    [body appendData:data];
    [body appendData:[[NSString stringWithFormat:@"\r\n--%@--\r\n", boundary] dataUsingEncoding:NSUTF8StringEncoding]];
    [request setHTTPBody:body];
    
    self.activeDownload = [NSMutableData data];
    NSLog(@"AFRURLReqest. Upload url: %@", self.urlString);
    NSURLConnection *conn = [[NSURLConnection alloc] initWithRequest:request delegate:self];
    self.apiConnection = conn;
}



- (NSString *)serializeParameters:(NSDictionary *)parameters
{
    if (parameters == nil)
        return nil;
    NSMutableString *result = [NSMutableString string];
    for (NSString *paramKey in parameters) {
        if ([paramKey length] > 0) {
            id value = [parameters objectForKey:paramKey];
            NSString *encodedValue;
            if ([value isKindOfClass:[NSArray class]]) {
                encodedValue = [[value componentsJoinedByString:@","] freeSpaces];
            }
            else if ([value isEqual:[NSNull null]]) {
                encodedValue = @"null";
            }
            else {
                encodedValue = [[value description] freeSpaces];
            }
            NSString *paramFormat = ([result length] == 0 ? @"%@=%@" : @"&%@=%@");
            [result appendFormat:paramFormat, paramKey, encodedValue];
        }
    }    
    return result;
}



#pragma mark - NSURLConnectionDelegate

- (void)connection:(NSURLConnection *)connection
   didSendBodyData:(NSInteger)bytesWritten
 totalBytesWritten:(NSInteger)totalBytesWritten
totalBytesExpectedToWrite:(NSInteger)totalBytesExpectedToWrite
{
    float uploadProgress = (float)totalBytesWritten / (float)totalBytesExpectedToWrite;
    
    if ([_delegate respondsToSelector:@selector(request:uploadProgress:)])
        [_delegate request:self uploadProgress:uploadProgress];
}



- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response
{
    _totalDataLength = (float)[response expectedContentLength];
    _loadedDataLength = 0.0;
	[self.activeDownload setLength:0];
}



- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data
{
    if (_totalDataLength != (float)NSURLResponseUnknownLength) {
        _loadedDataLength += [data length];
        float downloadProgress = _loadedDataLength / _totalDataLength;
        
        if ([_delegate respondsToSelector:@selector(request:downloadProgress:)])
            [_delegate request:self downloadProgress:downloadProgress];
    }
    
	[self.activeDownload appendData:data];
}



- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{
	// Clear the activeDownload property to allow later attempts
    self.activeDownload = nil;
    // Release the connection now that it's finished
    self.apiConnection = nil;
	[[AFActivityIndicatorManager sharedActivityIndicatorManager] decreaseActivityCounter];
    _completeBlock(nil, error);
}



- (void)connectionDidFinishLoading:(NSURLConnection *)connection
{
	[[AFActivityIndicatorManager sharedActivityIndicatorManager] decreaseActivityCounter];
	self.apiConnection = nil;
    _completeBlock(self.activeDownload, nil);
	self.activeDownload = nil;
}

@end
